#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
usage: plants_update03.py [-h] [-o OUTFILENAME] [-f] [-v] infilename

synopsis:
  parse an XML file; modify an object; write-out the modified object tree.

positional arguments:
  infilename            Input XML file name

optional arguments:
  -h, --help            show this help message and exit
  -o OUTFILENAME, --outfilename OUTFILENAME
                        Name/path of file to write to (default is stdout)
  -f, --force           Force overwrite of output file without asking
  -v, --verbose         Print messages during actions.

examples:
  python plants_update03.py plants_doc.xml
  python plants_update03.py -v plants_doc.xml
"""

import sys
import os
import argparse
import plants01lib as gdslib
import lupa


Separator = '-' * 50


def print_separator_message(msg):
    print(Separator)
    print(msg)
    print(Separator)


def parse_and_update(opts):
    # parse the XML imput file; create the gDS object tree.
    plantCollection = gdslib.parse(
        opts.infilename,
        silence=(not opts.verbose),
    )
    #
    # Begin Lua tests
    #
    lua = lupa.LuaRuntime(unpack_returned_tuples=True)
    luamodule = lua.require('testlupa01')
    veg01 = plantCollection.vegetables.get_vegetable()[0]
    luamodule.show_vegetable(veg01)
    veg02 = luamodule.make_vegetable(gdslib, 'lettuce')
    plantCollection.vegetables.add_vegetable(veg02)
    luamodule.show_vegetable(veg02)
    print_separator_message('After adding 1 vegetable')
    plantCollection.export(sys.stdout, 0,)
    luamodule.add_vegetables(
        gdslib,
        plantCollection.vegetables,
        ['green beans', 'cucumber', 'bell pepper'])
    print_separator_message('After add_vegetables')
    plantCollection.export(sys.stdout, 0)
    luamodule.upcase_names(gdslib, plantCollection)
    print_separator_message('After upcase_names')
    plantCollection.export(sys.stdout, 0)
    luamodule.export_it(plantCollection, sys.stdout)
    #
    # End Lua tests
    #
    # write out the updated object tree.
    if opts.outfilename:
        if os.path.exists(opts.outfilename) and not opts.force:
            sys.exit(
                "Error: use option -f/--force to overwrite existing file.")
        else:
            with open(opts.outfilename, 'w') as outfile:
                outfile.write('<?xml version="1.0" encoding="utf-8"?>\n')
                plantCollection.export(outfile, 0)
    else:
        print_separator_message('Displaying updated generateDS object tree')
        sys.stdout.write('<?xml version="1.0" encoding="utf-8"?>\n')
        plantCollection.export(sys.stdout, 0)


def main():
    description = """\
synopsis:
  parse an XML file; modify an object; write-out the modified object tree.
"""
    epilog = """\

examples:
  python plants_update03.py plants_doc.xml
  python plants_update03.py -v plants_doc.xml
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "infilename",
        help="Input XML file name"
    )
    parser.add_argument(
        "-o", "--outfilename",
        action="store", type=str,
        help="Name/path of file to write to (default is stdout)",
    )
    parser.add_argument(
        "-f", "--force",
        action="store_true",
        help="Force overwrite of output file without asking",
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions.",
    )
    opts = parser.parse_args()
    parse_and_update(opts)


if __name__ == '__main__':
    main()
